@@ -1,8 +1,9 @@
 """mqtt-entity library."""
 
 from mqtt_entity.client import MQTTClient
-from mqtt_entity.device import MQTTBaseEntity, MQTTDevice, MQTTOrigin
+from mqtt_entity.device import MQTTDevice
 from mqtt_entity.entities import (
+    MQTTBaseEntity,
     MQTTBinarySensorEntity,
     MQTTDeviceTrigger,
     MQTTEntity,
@@ -24,7 +25,6 @@ __all__ = [
     "MQTTEntity",
     "MQTTLightEntity",
     "MQTTNumberEntity",
-    "MQTTOrigin",
     "MQTTRWEntity",
     "MQTTSelectEntity",
     "MQTTSensorEntity",
